@@ -10,7 +10,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack import _log
 from openstack.baremetal.v1 import _common
 from openstack.baremetal.v1 import allocation as _allocation
 from openstack.baremetal.v1 import chassis as _chassis
@@ -22,9 +21,6 @@ from openstack import proxy
 from openstack import utils
 
 
-_logger = _log.setup_logging('openstack')
-
-
 class Proxy(proxy.Proxy):
 
     retriable_status_codes = _common.RETRIABLE_STATUS_CODES
@@ -114,6 +110,18 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_chassis.Chassis, chassis, **attrs)
 
+    def patch_chassis(self, chassis, patch):
+        """Apply a JSON patch to the chassis.
+
+        :param chassis: The value can be the ID of a chassis or a
+            :class:`~openstack.baremetal.v1.chassis.Chassis` instance.
+        :param patch: JSON patch to apply.
+
+        :returns: The updated chassis.
+        :rtype: :class:`~openstack.baremetal.v1.chassis.Chassis`
+        """
+        return self._get_resource(_chassis.Chassis, chassis).patch(self, patch)
+
     def delete_chassis(self, chassis, ignore_missing=True):
         """Delete a chassis.
 
@@ -246,8 +254,8 @@ class Proxy(proxy.Proxy):
     def update_node(self, node, retry_on_conflict=True, **attrs):
         """Update a node.
 
-        :param chassis: Either the name or the ID of a node or an instance
-            of :class:`~openstack.baremetal.v1.node.Node`.
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
         :param bool retry_on_conflict: Whether to retry HTTP CONFLICT error.
             Most of the time it can be retried, since it is caused by the node
             being locked. However, when setting ``instance_id``, this is
@@ -261,6 +269,23 @@ class Proxy(proxy.Proxy):
         res = self._get_resource(_node.Node, node, **attrs)
         return res.commit(self, retry_on_conflict=retry_on_conflict)
 
+    def patch_node(self, node, patch, retry_on_conflict=True):
+        """Apply a JSON patch to the node.
+
+        :param node: The value can be the name or ID of a node or a
+            :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param patch: JSON patch to apply.
+        :param bool retry_on_conflict: Whether to retry HTTP CONFLICT error.
+            Most of the time it can be retried, since it is caused by the node
+            being locked. However, when setting ``instance_id``, this is
+            a normal code and should not be retried.
+
+        :returns: The updated node.
+        :rtype: :class:`~openstack.baremetal.v1.node.Node`
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.patch(self, patch, retry_on_conflict=retry_on_conflict)
+
     def set_node_provision_state(self, node, target, config_drive=None,
                                  clean_steps=None, rescue_password=None,
                                  wait=False, timeout=None):
@@ -338,10 +363,11 @@ class Proxy(proxy.Proxy):
             if not remaining:
                 return finished
 
-            _logger.debug('Still waiting for nodes %(nodes)s to reach state '
-                          '"%(target)s"',
-                          {'nodes': ', '.join(n.id for n in remaining),
-                           'target': expected_state})
+            self.log.debug(
+                'Still waiting for nodes %(nodes)s to reach state '
+                '"%(target)s"',
+                {'nodes': ', '.join(n.id for n in remaining),
+                 'target': expected_state})
 
     def set_node_power_state(self, node, target):
         """Run an action modifying node's power state.
@@ -399,6 +425,27 @@ class Proxy(proxy.Proxy):
         res = self._get_resource(_node.Node, node)
         return res.validate(self, required=required)
 
+    def set_node_maintenance(self, node, reason=None):
+        """Enable maintenance mode on the node.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :param reason: Optional reason for maintenance.
+        :return: This :class:`Node` instance.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.set_maintenance(self, reason)
+
+    def unset_node_maintenance(self, node):
+        """Disable maintenance mode on the node.
+
+        :param node: The value can be either the name or ID of a node or
+            a :class:`~openstack.baremetal.v1.node.Node` instance.
+        :return: This :class:`Node` instance.
+        """
+        res = self._get_resource(_node.Node, node)
+        return res.unset_maintenance(self)
+
     def delete_node(self, node, ignore_missing=True):
         """Delete a node.
 
@@ -506,7 +553,7 @@ class Proxy(proxy.Proxy):
     def update_port(self, port, **attrs):
         """Update a port.
 
-        :param chassis: Either the ID of a port or an instance
+        :param port: Either the ID of a port or an instance
             of :class:`~openstack.baremetal.v1.port.Port`.
         :param dict attrs: The attributes to update on the port represented
             by the ``port`` parameter.
@@ -516,6 +563,18 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_port.Port, port, **attrs)
 
+    def patch_port(self, port, patch):
+        """Apply a JSON patch to the port.
+
+        :param port: The value can be the ID of a port or a
+            :class:`~openstack.baremetal.v1.port.Port` instance.
+        :param patch: JSON patch to apply.
+
+        :returns: The updated port.
+        :rtype: :class:`~openstack.baremetal.v1.port.Port`
+        """
+        return self._get_resource(_port.Port, port).patch(self, patch)
+
     def delete_port(self, port, ignore_missing=True):
         """Delete a port.
 
@@ -617,7 +676,7 @@ class Proxy(proxy.Proxy):
     def update_port_group(self, port_group, **attrs):
         """Update a port group.
 
-        :param chassis: Either the name or the ID of a port group or
+        :param port_group: Either the name or the ID of a port group or
             an instance of
             :class:`~openstack.baremetal.v1.port_group.PortGroup`.
         :param dict attrs: The attributes to update on the port group
@@ -628,6 +687,19 @@ class Proxy(proxy.Proxy):
         """
         return self._update(_portgroup.PortGroup, port_group, **attrs)
 
+    def patch_port_group(self, port_group, patch):
+        """Apply a JSON patch to the port_group.
+
+        :param port_group: The value can be the ID of a port group or a
+            :class:`~openstack.baremetal.v1.port_group.PortGroup` instance.
+        :param patch: JSON patch to apply.
+
+        :returns: The updated port group.
+        :rtype: :class:`~openstack.baremetal.v1.port_group.PortGroup`
+        """
+        res = self._get_resource(_portgroup.PortGroup, port_group)
+        return res.patch(self, patch)
+
     def delete_port_group(self, port_group, ignore_missing=True):
         """Delete a port group.
 
