@@ -47,7 +47,7 @@ is the recommended way to go:
 If the application in question is a command line application that should also
 accept command line arguments, an `argparse.Namespace` can be passed to
 :func:`openstack.connect` that will have relevant arguments added to it and
-then subsequently consumed by the construtor:
+then subsequently consumed by the constructor:
 
 .. code-block:: python
 
@@ -164,11 +164,24 @@ import six
 from openstack import _log
 from openstack._meta import connection as _meta
 from openstack.cloud import openstackcloud as _cloud
+from openstack.cloud import _baremetal
+from openstack.cloud import _block_storage
+from openstack.cloud import _compute
+from openstack.cloud import _clustering
+from openstack.cloud import _coe
+from openstack.cloud import _dns
+from openstack.cloud import _floating_ip
+from openstack.cloud import _identity
+from openstack.cloud import _image
+from openstack.cloud import _network
+from openstack.cloud import _network_common
+from openstack.cloud import _object_store
+from openstack.cloud import _orchestration
+from openstack.cloud import _security_group
 from openstack import config as _config
 from openstack.config import cloud_region
 from openstack import exceptions
 from openstack import service_description
-from openstack import task_manager as _task_manager
 
 __all__ = [
     'from_config',
@@ -212,7 +225,22 @@ def from_config(cloud=None, config=None, options=None, **kwargs):
 
 
 class Connection(six.with_metaclass(_meta.ConnectionMeta,
-                                    _cloud._OpenStackCloudMixin)):
+                                    _cloud._OpenStackCloudMixin,
+                                    _baremetal.BaremetalCloudMixin,
+                                    _block_storage.BlockStorageCloudMixin,
+                                    _compute.ComputeCloudMixin,
+                                    _clustering.ClusteringCloudMixin,
+                                    _coe.CoeCloudMixin,
+                                    _dns.DnsCloudMixin,
+                                    _floating_ip.FloatingIPCloudMixin,
+                                    _identity.IdentityCloudMixin,
+                                    _image.ImageCloudMixin,
+                                    _network.NetworkCloudMixin,
+                                    _network_common.NetworkCommonCloudMixin,
+                                    _object_store.ObjectStoreCloudMixin,
+                                    _orchestration.OrchestrationCloudMixin,
+                                    _security_group.SecurityGroupCloudMixin
+                                    )):
 
     def __init__(self, cloud=None, config=None, session=None,
                  app_name=None, app_version=None,
@@ -259,10 +287,8 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             filtering instead of making list calls and filtering client-side.
             Default false.
         :param task_manager:
-            Task Manager to handle the execution of remote REST calls.
-            Defaults to None which causes a direct-action Task Manager to be
-            used.
-        :type manager: :class:`~openstack.task_manager.TaskManager`
+            Ignored. Exists for backwards compat during transition. Rate limit
+            parameters should be passed directly to the `rate_limit` parameter.
         :param rate_limit:
             Client-side rate limit, expressed in calls per second. The
             parameter can either be a single float, or it can be a dict with
@@ -271,7 +297,7 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
             rate-limiting is performed.
         :param kwargs: If a config is not provided, the rest of the parameters
             provided are assumed to be arguments to be passed to the
-            CloudRegion contructor.
+            CloudRegion constructor.
         """
         self.config = config
         self._extra_services = {}
@@ -286,6 +312,7 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
                     app_name=app_name, app_version=app_version,
                     load_yaml_config=False,
                     load_envvars=False,
+                    rate_limit=rate_limit,
                     **kwargs)
             else:
                 self.config = _config.get_cloud_region(
@@ -293,25 +320,30 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
                     app_name=app_name, app_version=app_version,
                     load_yaml_config=cloud is not None,
                     load_envvars=cloud is not None,
+                    rate_limit=rate_limit,
                     **kwargs)
 
-        if task_manager:
-            # If a TaskManager was passed in, don't start it, assume it's
-            # under the control of the calling context.
-            self.task_manager = task_manager
-        else:
-            self.task_manager = _task_manager.TaskManager(
-                self.config.full_name,
-                rate=rate_limit)
-            self.task_manager.start()
-
         self._session = None
         self._proxies = {}
         self.use_direct_get = use_direct_get
         self.strict_mode = strict
-        # Call the _OpenStackCloudMixin constructor while we work on
+        # Call the _*CloudMixin constructors while we work on
         # integrating things better.
         _cloud._OpenStackCloudMixin.__init__(self)
+        _baremetal.BaremetalCloudMixin.__init__(self)
+        _block_storage.BlockStorageCloudMixin.__init__(self)
+        _clustering.ClusteringCloudMixin.__init__(self)
+        _coe.CoeCloudMixin.__init__(self)
+        _compute.ComputeCloudMixin.__init__(self)
+        _dns.DnsCloudMixin.__init__(self)
+        _floating_ip.FloatingIPCloudMixin.__init__(self)
+        _identity.IdentityCloudMixin.__init__(self)
+        _image.ImageCloudMixin.__init__(self)
+        _network_common.NetworkCommonCloudMixin.__init__(self)
+        _network.NetworkCloudMixin.__init__(self)
+        _object_store.ObjectStoreCloudMixin.__init__(self)
+        _orchestration.OrchestrationCloudMixin.__init__(self)
+        _security_group.SecurityGroupCloudMixin.__init__(self)
 
     @property
     def session(self):
@@ -345,9 +377,18 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
         if isinstance(service, six.string_types):
             service = service_description.ServiceDescription(service)
 
-        # Register the proxy class with every known alias
+        # Directly invoke descriptor of the ServiceDescription
+        def getter(self):
+            return service.__get__(self, service)
+
+        # Register the ServiceDescription class (as property)
+        # with every known alias for a "runtime descriptor"
         for attr_name in service.all_types:
-            setattr(self, attr_name.replace('-', '_'), service)
+            setattr(
+                self.__class__,
+                attr_name.replace('-', '_'),
+                property(fget=getter)
+            )
 
     def authorize(self):
         """Authorize this Connection
@@ -370,7 +411,8 @@ class Connection(six.with_metaclass(_meta.ConnectionMeta,
 
     def close(self):
         """Release any resources held open."""
-        self.task_manager.stop()
+        if self.__pool_executor:
+            self.__pool_executor.shutdown()
 
     def __enter__(self):
         return self
