@@ -10,20 +10,18 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack.network import network_service
-from openstack import resource2 as resource
+from openstack import resource
 
 
 class QoSRuleType(resource.Resource):
     resource_key = 'rule_type'
     resources_key = 'rule_types'
     base_path = '/qos/rule-types'
-    service = network_service.NetworkService()
 
     # capabilities
     allow_create = False
-    allow_get = True
-    allow_update = False
+    allow_fetch = True
+    allow_commit = False
     allow_delete = False
     allow_list = True
 
