@@ -10,11 +10,83 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from openstack import _adapter
+try:
+    import simplejson
+    JSONDecodeError = simplejson.scanner.JSONDecodeError
+except ImportError:
+    JSONDecodeError = ValueError
+from six.moves import urllib
+
+from keystoneauth1 import adapter
+
+from openstack import _log
 from openstack import exceptions
 from openstack import resource
 
 
+def _extract_name(url, service_type=None):
+    '''Produce a key name to use in logging/metrics from the URL path.
+
+    We want to be able to logic/metric sane general things, so we pull
+    the url apart to generate names. The function returns a list because
+    there are two different ways in which the elements want to be combined
+    below (one for logging, one for statsd)
+
+    Some examples are likely useful:
+
+    /servers -> ['servers']
+    /servers/{id} -> ['servers']
+    /servers/{id}/os-security-groups -> ['servers', 'os-security-groups']
+    /v2.0/networks.json -> ['networks']
+    '''
+
+    url_path = urllib.parse.urlparse(url).path.strip()
+    # Remove / from the beginning to keep the list indexes of interesting
+    # things consistent
+    if url_path.startswith('/'):
+        url_path = url_path[1:]
+
+    # Special case for neutron, which puts .json on the end of urls
+    if url_path.endswith('.json'):
+        url_path = url_path[:-len('.json')]
+
+    url_parts = url_path.split('/')
+    if url_parts[-1] == 'detail':
+        # Special case detail calls
+        # GET /servers/detail
+        # returns ['servers', 'detail']
+        name_parts = url_parts[-2:]
+    else:
+        # Strip leading version piece so that
+        # GET /v2.0/networks
+        # returns ['networks']
+        if (url_parts[0]
+                and url_parts[0][0] == 'v'
+                and url_parts[0][1] and url_parts[0][1].isdigit()):
+            url_parts = url_parts[1:]
+        name_parts = []
+        # Pull out every other URL portion - so that
+        # GET /servers/{id}/os-security-groups
+        # returns ['servers', 'os-security-groups']
+        for idx in range(0, len(url_parts)):
+            if not idx % 2 and url_parts[idx]:
+                name_parts.append(url_parts[idx])
+
+    # Keystone Token fetching is a special case, so we name it "tokens"
+    if url_path.endswith('tokens'):
+        name_parts = ['tokens']
+
+    # Getting the root of an endpoint is doing version discovery
+    if not name_parts:
+        if service_type == 'object-store':
+            name_parts = ['account']
+        else:
+            name_parts = ['discovery']
+
+    # Strip out anything that's empty or None
+    return [part for part in name_parts if part]
+
+
 # The _check_resource decorator is used on Proxy methods to ensure that
 # the `actual` argument is in fact the type of the `expected` argument.
 # It does so under two cases:
@@ -39,7 +111,7 @@ def _check_resource(strict=False):
     return wrap
 
 
-class Proxy(_adapter.OpenStackSDKAdapter):
+class Proxy(adapter.Adapter):
     """Represents a service."""
 
     retriable_status_codes = None
@@ -49,12 +121,80 @@ class Proxy(_adapter.OpenStackSDKAdapter):
     ``<service-type>_status_code_retries``.
     """
 
-    def __init__(self, *args, **kwargs):
+    def __init__(
+            self,
+            session,
+            statsd_client=None, statsd_prefix=None,
+            prometheus_counter=None, prometheus_histogram=None,
+            *args, **kwargs):
         # NOTE(dtantsur): keystoneauth defaults retriable_status_codes to None,
         # override it with a class-level value.
         kwargs.setdefault('retriable_status_codes',
                           self.retriable_status_codes)
-        super(Proxy, self).__init__(*args, **kwargs)
+        super(Proxy, self).__init__(session=session, *args, **kwargs)
+        self._statsd_client = statsd_client
+        self._statsd_prefix = statsd_prefix
+        self._prometheus_counter = prometheus_counter
+        self._prometheus_histogram = prometheus_histogram
+        if self.service_type:
+            log_name = 'openstack.{0}'.format(self.service_type)
+        else:
+            log_name = 'openstack'
+        self.log = _log.setup_logging(log_name)
+
+    def request(
+            self, url, method, error_message=None,
+            raise_exc=False, connect_retries=1, *args, **kwargs):
+        response = super(Proxy, self).request(
+            url, method,
+            connect_retries=connect_retries, raise_exc=False,
+            **kwargs)
+        for h in response.history:
+            self._report_stats(h)
+        self._report_stats(response)
+        return response
+
+    def _report_stats(self, response):
+        if self._statsd_client:
+            self._report_stats_statsd(response)
+        if self._prometheus_counter and self._prometheus_histogram:
+            self._report_stats_prometheus(response)
+
+    def _report_stats_statsd(self, response):
+        name_parts = _extract_name(response.request.url, self.service_type)
+        key = '.'.join(
+            [self._statsd_prefix, self.service_type, response.request.method]
+            + name_parts)
+        self._statsd_client.timing(key, int(response.elapsed.seconds * 1000))
+        self._statsd_client.incr(key)
+
+    def _report_stats_prometheus(self, response):
+        labels = dict(
+            method=response.request.method,
+            endpoint=response.request.url,
+            service_type=self.service_type,
+            status_code=response.status_code,
+        )
+        self._prometheus_counter.labels(**labels).inc()
+        self._prometheus_histogram.labels(**labels).observe(
+            response.elapsed.seconds)
+
+    def _version_matches(self, version):
+        api_version = self.get_api_major_version()
+        if api_version:
+            return api_version[0] == version
+        return False
+
+    def _get_connection(self):
+        """Get the Connection object associated with this Proxy.
+
+        When the Session is created, a reference to the Connection is attached
+        to the ``_sdk_connection`` attribute. We also add a reference to it
+        directly on ourselves. Use one of them.
+        """
+        return getattr(
+            self, '_connection', getattr(
+                self.session, '_sdk_connection', None))
 
     def _get_resource(self, resource_type, value, **attrs):
         """Get a resource object to work on
@@ -69,16 +209,19 @@ class Proxy(_adapter.OpenStackSDKAdapter):
         :param path_args: A dict containing arguments for forming the request
                           URL, if needed.
         """
+        conn = self._get_connection()
         if value is None:
             # Create a bare resource
-            res = resource_type.new(**attrs)
+            res = resource_type.new(connection=conn, **attrs)
         elif (isinstance(value, dict)
               and not isinstance(value, resource.Resource)):
-            res = resource_type._from_munch(value)
+            res = resource_type._from_munch(
+                value, connection=conn)
             res._update(**attrs)
         elif not isinstance(value, resource_type):
             # Create from an ID
-            res = resource_type.new(id=value, **attrs)
+            res = resource_type.new(
+                id=value, connection=conn, **attrs)
         else:
             # An existing resource instance
             res = value
@@ -205,7 +348,8 @@ class Proxy(_adapter.OpenStackSDKAdapter):
         :returns: The result of the ``create``
         :rtype: :class:`~openstack.resource.Resource`
         """
-        res = resource_type.new(**attrs)
+        conn = self._get_connection()
+        res = resource_type.new(connection=conn, **attrs)
         return res.create(self, base_path=base_path)
 
     @_check_resource(strict=False)
@@ -239,7 +383,7 @@ class Proxy(_adapter.OpenStackSDKAdapter):
                 resource_type=resource_type.__name__, value=value))
 
     def _list(self, resource_type, value=None,
-              paginated=False, base_path=None, **attrs):
+              paginated=True, base_path=None, **attrs):
         """List a resource
 
         :param resource_type: The type of resource to delete. This should
@@ -265,9 +409,10 @@ class Proxy(_adapter.OpenStackSDKAdapter):
                  :class:`~openstack.resource.Resource` that doesn't match
                  the ``resource_type``.
         """
-        res = self._get_resource(resource_type, value, **attrs)
-        return res.list(self, paginated=paginated,
-                        base_path=base_path, **attrs)
+        return resource_type.list(
+            self, paginated=paginated,
+            base_path=base_path,
+            **attrs)
 
     def _head(self, resource_type, value=None, base_path=None, **attrs):
         """Retrieve a resource's header
@@ -291,3 +436,30 @@ class Proxy(_adapter.OpenStackSDKAdapter):
         """
         res = self._get_resource(resource_type, value, **attrs)
         return res.head(self, base_path=base_path)
+
+
+def _json_response(response, result_key=None, error_message=None):
+    """Temporary method to use to bridge from ShadeAdapter to SDK calls."""
+    exceptions.raise_from_response(response, error_message=error_message)
+
+    if not response.content:
+        # This doesn't have any content
+        return response
+
+    # Some REST calls do not return json content. Don't decode it.
+    if 'application/json' not in response.headers.get('Content-Type'):
+        return response
+
+    try:
+        result_json = response.json()
+    except JSONDecodeError:
+        return response
+    return result_json
+
+
+class _ShadeAdapter(Proxy):
+    """Wrapper for shade methods that expect json unpacking."""
+
+    def request(self, url, method, error_message=None, **kwargs):
+        response = super(_ShadeAdapter, self).request(url, method, **kwargs)
+        return _json_response(response, error_message=error_message)
