@@ -35,6 +35,7 @@ import collections
 import itertools
 
 import jsonpatch
+import operator
 from keystoneauth1 import adapter
 from keystoneauth1 import discover
 import munch
@@ -387,6 +388,8 @@ class Resource(dict):
     allow_list = False
     #: Allow head operation for this resource.
     allow_head = False
+    #: Allow patch operation for this resource.
+    allow_patch = False
 
     # TODO(mordred) Unused - here for transition with OSC. Remove once
     # OSC no longer checks for allow_get
@@ -404,8 +407,12 @@ class Resource(dict):
 
     #: Do calls for this resource require an id
     requires_id = True
+    #: Whether create requires an ID (determined from method if None).
+    create_requires_id = None
     #: Do responses for this resource have bodies
     has_body = True
+    #: Does create returns a body (if False requires ID), defaults to has_body
+    create_returns_body = None
 
     #: Maximum microversion to use for getting/creating/updating the Resource
     _max_microversion = None
@@ -419,6 +426,7 @@ class Resource(dict):
     _computed = None
     _original_body = None
     _delete_response_class = None
+    _store_unknown_attrs_as_properties = False
 
     def __init__(self, _synchronized=False, connection=None, **attrs):
         """The base resource
@@ -439,9 +447,6 @@ class Resource(dict):
         # items as they match up with any of the body, header,
         # or uri mappings.
         body, header, uri, computed = self._collect_attrs(attrs)
-        # TODO(briancurtin): at this point if attrs has anything left
-        # they're not being set anywhere. Log this? Raise exception?
-        # How strict should we be here? Should strict be an option?
 
         self._body = _ComponentManager(
             attributes=body,
@@ -455,7 +460,7 @@ class Resource(dict):
         self._computed = _ComponentManager(
             attributes=computed,
             synchronized=_synchronized)
-        if self.commit_jsonpatch:
+        if self.commit_jsonpatch or self.allow_patch:
             # We need the original body to compare against
             if _synchronized:
                 self._original_body = self._body.attributes.copy()
@@ -466,6 +471,13 @@ class Resource(dict):
                 }
             else:
                 self._original_body = {}
+        if self._store_unknown_attrs_as_properties:
+            # When storing of unknown attributes is requested - ensure
+            # we have properties attribute (with type=None)
+            self._store_unknown_attrs_as_properties = (
+                hasattr(self.__class__, 'properties')
+                and self.__class__.properties.type is None
+            )
 
         self._update_location()
 
@@ -498,7 +510,7 @@ class Resource(dict):
             self._body.attributes == comparand._body.attributes,
             self._header.attributes == comparand._header.attributes,
             self._uri.attributes == comparand._uri.attributes,
-            self._computed.attributes == comparand._computed.attributes,
+            self._computed.attributes == comparand._computed.attributes
         ])
 
     def __getattribute__(self, name):
@@ -596,6 +608,10 @@ class Resource(dict):
         header = self._consume_header_attrs(attrs)
         uri = self._consume_uri_attrs(attrs)
 
+        if attrs and self._store_unknown_attrs_as_properties:
+            # Keep also remaining (unknown) attributes
+            body = self._pack_attrs_under_properties(body, attrs)
+
         if any([body, header, uri]):
             attrs = self._compute_attributes(body, header, uri)
 
@@ -695,6 +711,14 @@ class Resource(dict):
 
         return relevant_attrs
 
+    def _clean_body_attrs(self, attrs):
+        """Mark the attributes as up-to-date."""
+        self._body.clean(only=attrs)
+        if self.commit_jsonpatch or self.allow_patch:
+            for attr in attrs:
+                if attr in self._body:
+                    self._original_body[attr] = self._body[attr]
+
     @classmethod
     def _get_mapping(cls, component):
         """Return a dict of attributes of a given component on the class"""
@@ -897,12 +921,55 @@ class Resource(dict):
             body=True, headers=False,
             original_names=original_names, _to_munch=True)
 
+    def _unpack_properties_to_resource_root(self, body):
+        if not body:
+            return
+        # We do not want to modify caller
+        body = body.copy()
+        props = body.pop('properties', {})
+        if props and isinstance(props, dict):
+            # unpack dict of properties back to the root of the resource
+            body.update(props)
+        elif props and isinstance(props, str):
+            # A string value only - bring it back
+            body['properties'] = props
+        return body
+
+    def _pack_attrs_under_properties(self, body, attrs):
+        props = body.get('properties', {})
+        if not isinstance(props, dict):
+            props = {'properties': props}
+        props.update(attrs)
+        body['properties'] = props
+        return body
+
     def _prepare_request_body(self, patch, prepend_key):
         if patch:
-            new = self._body.attributes
-            body = jsonpatch.make_patch(self._original_body, new).patch
+            if not self._store_unknown_attrs_as_properties:
+                # Default case
+                new = self._body.attributes
+                original_body = self._original_body
+            else:
+                new = self._unpack_properties_to_resource_root(
+                    self._body.attributes)
+                original_body = self._unpack_properties_to_resource_root(
+                    self._original_body)
+
+            # NOTE(gtema) sort result, since we might need validate it in tests
+            body = sorted(
+                list(jsonpatch.make_patch(
+                    original_body,
+                    new).patch),
+                key=operator.itemgetter('path')
+            )
         else:
-            body = self._body.dirty
+            if not self._store_unknown_attrs_as_properties:
+                # Default case
+                body = self._body.dirty
+            else:
+                body = self._unpack_properties_to_resource_root(
+                    self._body.dirty)
+
             if prepend_key and self.resource_key is not None:
                 body = {self.resource_key: body}
         return body
@@ -961,16 +1028,26 @@ class Resource(dict):
             has_body = self.has_body
         exceptions.raise_from_response(response, error_message=error_message)
         if has_body:
-            body = response.json()
-            if self.resource_key and self.resource_key in body:
-                body = body[self.resource_key]
+            try:
+                body = response.json()
+                if self.resource_key and self.resource_key in body:
+                    body = body[self.resource_key]
 
-            body = self._consume_body_attrs(body)
-            self._body.attributes.update(body)
-            self._body.clean()
-            if self.commit_jsonpatch:
-                # We need the original body to compare against
-                self._original_body = body.copy()
+                body_attrs = self._consume_body_attrs(body)
+
+                if self._store_unknown_attrs_as_properties:
+                    body_attrs = self._pack_attrs_under_properties(
+                        body_attrs, body)
+
+                self._body.attributes.update(body_attrs)
+                self._body.clean()
+                if self.commit_jsonpatch or self.allow_patch:
+                    # We need the original body to compare against
+                    self._original_body = body_attrs.copy()
+            except ValueError:
+                # Server returned not parse-able response (202, 204, etc)
+                # Do simply nothing
+                pass
 
         headers = self._consume_header_attrs(response.headers)
         self._header.attributes.update(headers)
@@ -1029,11 +1106,11 @@ class Resource(dict):
         Subclasses can override this method if more complex logic is needed.
 
         :param session: :class`keystoneauth1.adapter.Adapter`
-        :param action: One of "fetch", "commit", "create", "delete". Unused in
-            the base implementation.
+        :param action: One of "fetch", "commit", "create", "delete", "patch".
+            Unused in the base implementation.
         :return: microversion as string or ``None``
         """
-        if action not in ('fetch', 'commit', 'create', 'delete'):
+        if action not in ('fetch', 'commit', 'create', 'delete', 'patch'):
             raise ValueError('Invalid action: %s' % action)
 
         return self._get_microversion_for_list(session)
@@ -1094,15 +1171,18 @@ class Resource(dict):
 
         session = self._get_session(session)
         microversion = self._get_microversion_for(session, 'create')
+        requires_id = (self.create_requires_id
+                       if self.create_requires_id is not None
+                       else self.create_method == 'PUT')
         if self.create_method == 'PUT':
-            request = self._prepare_request(requires_id=True,
+            request = self._prepare_request(requires_id=requires_id,
                                             prepend_key=prepend_key,
                                             base_path=base_path)
             response = session.put(request.url,
                                    json=request.body, headers=request.headers,
                                    microversion=microversion)
         elif self.create_method == 'POST':
-            request = self._prepare_request(requires_id=False,
+            request = self._prepare_request(requires_id=requires_id,
                                             prepend_key=prepend_key,
                                             base_path=base_path)
             response = session.post(request.url,
@@ -1112,12 +1192,18 @@ class Resource(dict):
             raise exceptions.ResourceFailure(
                 msg="Invalid create method: %s" % self.create_method)
 
+        has_body = (self.has_body if self.create_returns_body is None
+                    else self.create_returns_body)
         self.microversion = microversion
-        self._translate_response(response)
+        self._translate_response(response, has_body=has_body)
+        # direct comparision to False since we need to rule out None
+        if self.has_body and self.create_returns_body is False:
+            # fetch the body if it's required but not returned by create
+            return self.fetch(session)
         return self
 
     def fetch(self, session, requires_id=True,
-              base_path=None, error_message=None):
+              base_path=None, error_message=None, **params):
         """Get a remote resource based on this instance.
 
         :param session: The session to use for making this request.
@@ -1129,6 +1215,7 @@ class Resource(dict):
                               :data:`~openstack.resource.Resource.base_path`.
         :param str error_message: An Error message to be returned if
                                   requested object does not exist.
+        :param dict params: Additional parameters that can be consumed.
         :return: This :class:`Resource` instance.
         :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
                  :data:`Resource.allow_fetch` is not set to ``True``.
@@ -1181,6 +1268,12 @@ class Resource(dict):
         self._translate_response(response, has_body=False)
         return self
 
+    @property
+    def requires_commit(self):
+        """Whether the next commit() call will do anything."""
+        return (self._body.dirty or self._header.dirty
+                or self.allow_empty_commit)
+
     def commit(self, session, prepend_key=True, has_body=True,
                retry_on_conflict=None, base_path=None):
         """Commit the state of the instance to the remote resource.
@@ -1205,11 +1298,7 @@ class Resource(dict):
         self._body._dirty.discard("id")
 
         # Only try to update if we actually have anything to commit.
-        if not any([
-            self._body.dirty,
-            self._header.dirty,
-            self.allow_empty_commit,
-        ]):
+        if not self.requires_commit:
             return self
 
         if not self.allow_commit:
@@ -1224,6 +1313,14 @@ class Resource(dict):
         request = self._prepare_request(prepend_key=prepend_key,
                                         base_path=base_path,
                                         **kwargs)
+        microversion = self._get_microversion_for(session, 'commit')
+
+        return self._commit(session, request, self.commit_method, microversion,
+                            has_body=has_body,
+                            retry_on_conflict=retry_on_conflict)
+
+    def _commit(self, session, request, method, microversion, has_body=True,
+                retry_on_conflict=None):
         session = self._get_session(session)
 
         kwargs = {}
@@ -1235,28 +1332,95 @@ class Resource(dict):
             # overriding it via an explicit retry_on_conflict=False.
             kwargs['retriable_status_codes'] = retriable_status_codes - {409}
 
-        microversion = self._get_microversion_for(session, 'commit')
-
-        if self.commit_method == 'PATCH':
-            response = session.patch(
-                request.url, json=request.body, headers=request.headers,
-                microversion=microversion, **kwargs)
-        elif self.commit_method == 'POST':
-            response = session.post(
-                request.url, json=request.body, headers=request.headers,
-                microversion=microversion, **kwargs)
-        elif self.commit_method == 'PUT':
-            response = session.put(
-                request.url, json=request.body, headers=request.headers,
-                microversion=microversion, **kwargs)
-        else:
+        try:
+            call = getattr(session, method.lower())
+        except AttributeError:
             raise exceptions.ResourceFailure(
-                msg="Invalid commit method: %s" % self.commit_method)
+                msg="Invalid commit method: %s" % method)
+
+        response = call(request.url, json=request.body,
+                        headers=request.headers, microversion=microversion,
+                        **kwargs)
 
         self.microversion = microversion
         self._translate_response(response, has_body=has_body)
         return self
 
+    def _convert_patch(self, patch):
+        if not isinstance(patch, list):
+            patch = [patch]
+
+        converted = []
+        for item in patch:
+            try:
+                path = item['path']
+                parts = path.lstrip('/').split('/', 1)
+                field = parts[0]
+            except (KeyError, IndexError):
+                raise ValueError("Malformed or missing path in %s" % item)
+
+            try:
+                component = getattr(self.__class__, field)
+            except AttributeError:
+                server_field = field
+            else:
+                server_field = component.name
+
+            if len(parts) > 1:
+                new_path = '/%s/%s' % (server_field, parts[1])
+            else:
+                new_path = '/%s' % server_field
+            converted.append(dict(item, path=new_path))
+
+        return converted
+
+    def patch(self, session, patch=None, prepend_key=True, has_body=True,
+              retry_on_conflict=None, base_path=None):
+        """Patch the remote resource.
+
+        Allows modifying the resource by providing a list of JSON patches to
+        apply to it. The patches can use both the original (server-side) and
+        SDK field names.
+
+        :param session: The session to use for making this request.
+        :type session: :class:`~keystoneauth1.adapter.Adapter`
+        :param patch: Additional JSON patch as a list or one patch item.
+                      If provided, it is applied on top of any changes to the
+                      current resource.
+        :param prepend_key: A boolean indicating whether the resource_key
+                            should be prepended in a resource update request.
+                            Default to True.
+        :param bool retry_on_conflict: Whether to enable retries on HTTP
+                                       CONFLICT (409). Value of ``None`` leaves
+                                       the `Adapter` defaults.
+        :param str base_path: Base part of the URI for modifying resources, if
+                              different from
+                              :data:`~openstack.resource.Resource.base_path`.
+
+        :return: This :class:`Resource` instance.
+        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
+                 :data:`Resource.allow_patch` is not set to ``True``.
+        """
+        # The id cannot be dirty for an commit
+        self._body._dirty.discard("id")
+
+        # Only try to update if we actually have anything to commit.
+        if not patch and not self.requires_commit:
+            return self
+
+        if not self.allow_patch:
+            raise exceptions.MethodNotSupported(self, "patch")
+
+        request = self._prepare_request(prepend_key=prepend_key,
+                                        base_path=base_path, patch=True)
+        microversion = self._get_microversion_for(session, 'patch')
+        if patch:
+            request.body += self._convert_patch(patch)
+
+        return self._commit(session, request, 'PATCH', microversion,
+                            has_body=has_body,
+                            retry_on_conflict=retry_on_conflict)
+
     def delete(self, session, error_message=None):
         """Delete the remote resource based on this instance.
 
@@ -1429,6 +1593,17 @@ class Resource(dict):
                     if limit:
                         params['limit'] = limit
                     next_link = uri
+
+        # Parse params from Link (next page URL) into params.
+        # This prevents duplication of query parameters that with large
+        # number of pages result in HTTP 414 error eventually.
+        if next_link:
+            parts = six.moves.urllib.parse.urlparse(next_link)
+            query_params = six.moves.urllib.parse.parse_qs(parts.query)
+            params.update(query_params)
+            next_link = six.moves.urllib.parse.urljoin(next_link,
+                                                       parts.path)
+
         # If we still have no link, and limit was given and is non-zero,
         # and the number of records yielded equals the limit, then the user
         # is playing pagination ball so we should go ahead and try once more.
@@ -1490,10 +1665,14 @@ class Resource(dict):
                 id=name_or_id,
                 connection=session._get_connection(),
                 **params)
-            return match.fetch(session)
+            return match.fetch(session, **params)
         except exceptions.NotFoundException:
             pass
 
+        if ('name' in cls._query_mapping._mapping.keys()
+                and 'name' not in params):
+            params['name'] = name_or_id
+
         data = cls.list(session, **params)
 
         result = cls._get_one_match(name_or_id, data)
