@@ -15,8 +15,8 @@ import uuid
 
 import testtools
 
-import openstack.cloud
 from openstack.cloud import exc
+from openstack import connection
 from openstack.tests import fakes
 from openstack.tests.unit import base
 from openstack import utils
@@ -52,7 +52,7 @@ class TestShade(base.TestCase):
         self.cloud.has_service = fake_has_service
 
     def test_openstack_cloud(self):
-        self.assertIsInstance(self.cloud, openstack.cloud.OpenStackCloud)
+        self.assertIsInstance(self.cloud, connection.Connection)
 
     def test_connect_as(self):
         # Do initial auth/catalog steps
@@ -62,7 +62,7 @@ class TestShade(base.TestCase):
         # keystoneauth1.loading.base.BaseLoader.load_from_options
         self.cloud.connect_as(project_name='test_project')
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'search_images')
+    @mock.patch.object(connection.Connection, 'search_images')
     def test_get_images(self, mock_search):
         image1 = dict(id='123', name='mickey')
         mock_search.return_value = [image1]
@@ -70,7 +70,7 @@ class TestShade(base.TestCase):
         self.assertIsNotNone(r)
         self.assertDictEqual(image1, r)
 
-    @mock.patch.object(openstack.cloud.OpenStackCloud, 'search_images')
+    @mock.patch.object(connection.Connection, 'search_images')
     def test_get_image_not_found(self, mock_search):
         mock_search.return_value = []
         r = self.cloud.get_image('doesNotExist')
@@ -81,6 +81,7 @@ class TestShade(base.TestCase):
         server2 = fakes.make_fake_server('345', 'mouse')
 
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -95,6 +96,7 @@ class TestShade(base.TestCase):
 
     def test_get_server_not_found(self):
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -108,6 +110,7 @@ class TestShade(base.TestCase):
 
     def test_list_servers_exception(self):
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -146,6 +149,7 @@ class TestShade(base.TestCase):
         server_name = self.getUniqueString('name')
         fake_server = fakes.make_fake_server(server_id, server_name)
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -329,6 +333,7 @@ class TestShade(base.TestCase):
                     "name": "private-subnet-ipv4"
                 }]}
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail']),
@@ -353,6 +358,7 @@ class TestShade(base.TestCase):
         '''This test verifies that when list_servers is called with
         `all_projects=True` that it passes `all_tenants=True` to nova.'''
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail'],
@@ -369,6 +375,7 @@ class TestShade(base.TestCase):
         '''This test verifies that when list_servers is called with
         `filters` dict that it passes it to nova.'''
         self.register_uris([
+            self.get_nova_discovery_mock_dict(),
             dict(method='GET',
                  uri=self.get_mock_url(
                      'compute', 'public', append=['servers', 'detail'],
