@@ -28,6 +28,7 @@ EXAMPLE = {
     'protocol_port': 5,
     'subnet_id': uuid.uuid4(),
     'weight': 7,
+    'backup': False,
 }
 
 
@@ -37,12 +38,11 @@ class TestPoolMember(base.TestCase):
         test_member = member.Member()
         self.assertEqual('member', test_member.resource_key)
         self.assertEqual('members', test_member.resources_key)
-        self.assertEqual('/v2.0/lbaas/pools/%(pool_id)s/members',
+        self.assertEqual('/lbaas/pools/%(pool_id)s/members',
                          test_member.base_path)
-        self.assertEqual('load-balancer', test_member.service.service_type)
         self.assertTrue(test_member.allow_create)
-        self.assertTrue(test_member.allow_get)
-        self.assertTrue(test_member.allow_update)
+        self.assertTrue(test_member.allow_fetch)
+        self.assertTrue(test_member.allow_commit)
         self.assertTrue(test_member.allow_delete)
         self.assertTrue(test_member.allow_list)
 
@@ -60,3 +60,29 @@ class TestPoolMember(base.TestCase):
         self.assertEqual(EXAMPLE['protocol_port'], test_member.protocol_port)
         self.assertEqual(EXAMPLE['subnet_id'], test_member.subnet_id)
         self.assertEqual(EXAMPLE['weight'], test_member.weight)
+        self.assertFalse(test_member.backup)
+
+        self.assertDictEqual(
+            {'limit': 'limit',
+             'marker': 'marker',
+             'created_at': 'created_at',
+             'updated_at': 'updated_at',
+             'name': 'name',
+             'project_id': 'project_id',
+             'tags': 'tags',
+             'any_tags': 'tags-any',
+             'not_tags': 'not-tags',
+             'not_any_tags': 'not-tags-any',
+             'operating_status': 'operating_status',
+             'provisioning_status': 'provisioning_status',
+             'is_admin_state_up': 'admin_state_up',
+
+             'address': 'address',
+             'protocol_port': 'protocol_port',
+             'subnet_id': 'subnet_id',
+             'weight': 'weight',
+             'monitor_address': 'monitor_address',
+             'monitor_port': 'monitor_port',
+             'backup': 'backup'
+             },
+            test_member._query_mapping._mapping)
