@@ -40,13 +40,40 @@ class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
                            method_kwargs=method_kwargs)
 
     def test_find_stack(self):
-        self.verify_find(self.proxy.find_stack, stack.Stack)
+        self.verify_find(self.proxy.find_stack, stack.Stack,
+                         expected_kwargs={'resolve_outputs': True})
+        # mock_method="openstack.proxy.Proxy._find"
+        # test_method=self.proxy.find_stack
+        # method_kwargs = {
+        #     'resolve_outputs': False,
+        #     'ignore_missing': False
+        # }
+        # method_args=["name_or_id"]
+        # self._verify2(mock_method, test_method,
+        #               method_args=method_args,
+        #               method_kwargs=method_kwargs,
+        #               expected_args=[stack.Stack, "name_or_id"],
+        #               expected_kwargs=method_kwargs,
+        #               expected_result="result")
+        #
+        # method_kwargs = {
+        #     'resolve_outputs': True,
+        #     'ignore_missing': True
+        # }
+        # self._verify2(mock_method, test_method,
+        #               method_args=method_args,
+        #               method_kwargs=method_kwargs,
+        #               expected_args=[stack.Stack, "name_or_id"],
+        #               expected_kwargs=method_kwargs,
+        #               expected_result="result")
 
     def test_stacks(self):
-        self.verify_list(self.proxy.stacks, stack.Stack, paginated=False)
+        self.verify_list(self.proxy.stacks, stack.Stack)
 
     def test_get_stack(self):
-        self.verify_get(self.proxy.get_stack, stack.Stack)
+        self.verify_get(self.proxy.get_stack, stack.Stack,
+                        method_kwargs={'resolve_outputs': False},
+                        expected_kwargs={'resolve_outputs': False})
         self.verify_get_overrided(
             self.proxy, stack.Stack,
             'openstack.orchestration.v1.stack.Stack')
@@ -195,7 +222,7 @@ class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
         stk = stack.Stack(id=stack_id, name=stack_name)
 
         self.verify_list(self.proxy.resources, resource.Resource,
-                         paginated=False, method_args=[stk],
+                         method_args=[stk],
                          expected_kwargs={'stack_name': stack_name,
                                           'stack_id': stack_id})
 
@@ -209,7 +236,7 @@ class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
         mock_find.return_value = stk
 
         self.verify_list(self.proxy.resources, resource.Resource,
-                         paginated=False, method_args=[stack_id],
+                         method_args=[stack_id],
                          expected_kwargs={'stack_name': stack_name,
                                           'stack_id': stack_id})
 
@@ -232,8 +259,7 @@ class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
                            sc.SoftwareConfig)
 
     def test_software_configs(self):
-        self.verify_list(self.proxy.software_configs, sc.SoftwareConfig,
-                         paginated=True)
+        self.verify_list(self.proxy.software_configs, sc.SoftwareConfig)
 
     def test_get_software_config(self):
         self.verify_get(self.proxy.get_software_config, sc.SoftwareConfig)
@@ -250,7 +276,7 @@ class TestOrchestrationProxy(test_proxy_base.TestProxyBase):
 
     def test_software_deployments(self):
         self.verify_list(self.proxy.software_deployments,
-                         sd.SoftwareDeployment, paginated=False)
+                         sd.SoftwareDeployment)
 
     def test_get_software_deployment(self):
         self.verify_get(self.proxy.get_software_deployment,
