@@ -9,6 +9,7 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from testscenarios import load_tests_apply_scenarios as load_tests  # noqa
 
 import mock
 import munch
@@ -24,7 +25,7 @@ class DeleteableResource(resource.Resource):
 
 
 class UpdateableResource(resource.Resource):
-    allow_update = True
+    allow_commit = True
 
 
 class CreateableResource(resource.Resource):
@@ -54,7 +55,10 @@ class TestProxyPrivate(base.TestCase):
         self.sot = mock.Mock()
         self.sot.method = method
 
-        self.fake_proxy = proxy.Proxy("session")
+        self.session = mock.Mock()
+        self.session._sdk_connection = self.cloud
+        self.fake_proxy = proxy.Proxy(self.session)
+        self.fake_proxy._connection = self.cloud
 
     def _test_correct(self, value):
         decorated = proxy._check_resource(strict=False)(self.sot.method)
@@ -119,7 +123,7 @@ class TestProxyPrivate(base.TestCase):
 
         result = self.fake_proxy._get_resource(fake_type, None, **attrs)
 
-        fake_type.new.assert_called_with(**attrs)
+        fake_type.new.assert_called_with(connection=self.cloud, **attrs)
         self.assertEqual(value, result)
 
     def test__get_resource_from_id(self):
@@ -143,7 +147,8 @@ class TestProxyPrivate(base.TestCase):
 
         result = self.fake_proxy._get_resource(Fake, id, **attrs)
 
-        self.assertDictEqual(dict(id=id, **attrs), Fake.call)
+        self.assertDictEqual(
+            dict(id=id, connection=mock.ANY, **attrs), Fake.call)
         self.assertEqual(value, result)
 
     def test__get_resource_from_resource(self):
@@ -169,7 +174,7 @@ class TestProxyPrivate(base.TestCase):
 
         result = self.fake_proxy._get_resource(cls, m, **attrs)
 
-        cls._from_munch.assert_called_once_with(m)
+        cls._from_munch.assert_called_once_with(m, connection=self.cloud)
         res._update.assert_called_once_with(**attrs)
         self.assertEqual(result, res)
 
@@ -180,6 +185,7 @@ class TestProxyDelete(base.TestCase):
         super(TestProxyDelete, self).setUp()
 
         self.session = mock.Mock()
+        self.session._sdk_connection = self.cloud
 
         self.fake_id = 1
         self.res = mock.Mock(spec=DeleteableResource)
@@ -187,6 +193,7 @@ class TestProxyDelete(base.TestCase):
         self.res.delete = mock.Mock()
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
         DeleteableResource.new = mock.Mock(return_value=self.res)
 
     def test_delete(self):
@@ -194,7 +201,8 @@ class TestProxyDelete(base.TestCase):
         self.res.delete.assert_called_with(self.sot)
 
         self.sot._delete(DeleteableResource, self.fake_id)
-        DeleteableResource.new.assert_called_with(id=self.fake_id)
+        DeleteableResource.new.assert_called_with(
+            connection=self.cloud, id=self.fake_id)
         self.res.delete.assert_called_with(self.sot)
 
         # Delete generally doesn't return anything, so we will normally
@@ -205,18 +213,18 @@ class TestProxyDelete(base.TestCase):
         self.assertEqual(rv, self.fake_id)
 
     def test_delete_ignore_missing(self):
-        self.res.delete.side_effect = exceptions.NotFoundException(
+        self.res.delete.side_effect = exceptions.ResourceNotFound(
             message="test", http_status=404)
 
         rv = self.sot._delete(DeleteableResource, self.fake_id)
         self.assertIsNone(rv)
 
     def test_delete_NotFound(self):
-        self.res.delete.side_effect = exceptions.NotFoundException(
+        self.res.delete.side_effect = exceptions.ResourceNotFound(
             message="test", http_status=404)
 
         self.assertRaisesRegex(
-            exceptions.NotFoundException,
+            exceptions.ResourceNotFound,
             # TODO(shade) The mocks here are hiding the thing we want to test.
             "test",
             self.sot._delete, DeleteableResource, self.res,
@@ -241,9 +249,10 @@ class TestProxyUpdate(base.TestCase):
         self.fake_result = "fake_result"
 
         self.res = mock.Mock(spec=UpdateableResource)
-        self.res.update = mock.Mock(return_value=self.fake_result)
+        self.res.commit = mock.Mock(return_value=self.fake_result)
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
 
         self.attrs = {"x": 1, "y": 2, "z": 3}
 
@@ -254,13 +263,22 @@ class TestProxyUpdate(base.TestCase):
 
         self.assertEqual(rv, self.fake_result)
         self.res._update.assert_called_once_with(**self.attrs)
-        self.res.update.assert_called_once_with(self.sot)
+        self.res.commit.assert_called_once_with(self.sot, base_path=None)
+
+    def test_update_resource_override_base_path(self):
+        base_path = 'dummy'
+        rv = self.sot._update(UpdateableResource, self.res,
+                              base_path=base_path, **self.attrs)
+
+        self.assertEqual(rv, self.fake_result)
+        self.res._update.assert_called_once_with(**self.attrs)
+        self.res.commit.assert_called_once_with(self.sot, base_path=base_path)
 
     def test_update_id(self):
         rv = self.sot._update(UpdateableResource, self.fake_id, **self.attrs)
 
         self.assertEqual(rv, self.fake_result)
-        self.res.update.assert_called_once_with(self.sot)
+        self.res.commit.assert_called_once_with(self.sot, base_path=None)
 
 
 class TestProxyCreate(base.TestCase):
@@ -269,12 +287,14 @@ class TestProxyCreate(base.TestCase):
         super(TestProxyCreate, self).setUp()
 
         self.session = mock.Mock()
+        self.session._sdk_connection = self.cloud
 
         self.fake_result = "fake_result"
         self.res = mock.Mock(spec=CreateableResource)
         self.res.create = mock.Mock(return_value=self.fake_result)
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
 
     def test_create_attributes(self):
         CreateableResource.new = mock.Mock(return_value=self.res)
@@ -283,8 +303,21 @@ class TestProxyCreate(base.TestCase):
         rv = self.sot._create(CreateableResource, **attrs)
 
         self.assertEqual(rv, self.fake_result)
-        CreateableResource.new.assert_called_once_with(**attrs)
-        self.res.create.assert_called_once_with(self.sot)
+        CreateableResource.new.assert_called_once_with(
+            connection=self.cloud, **attrs)
+        self.res.create.assert_called_once_with(self.sot, base_path=None)
+
+    def test_create_attributes_override_base_path(self):
+        CreateableResource.new = mock.Mock(return_value=self.res)
+
+        base_path = 'dummy'
+        attrs = {"x": 1, "y": 2, "z": 3}
+        rv = self.sot._create(CreateableResource, base_path=base_path, **attrs)
+
+        self.assertEqual(rv, self.fake_result)
+        CreateableResource.new.assert_called_once_with(
+            connection=self.cloud, **attrs)
+        self.res.create.assert_called_once_with(self.sot, base_path=base_path)
 
 
 class TestProxyGet(base.TestCase):
@@ -293,22 +326,26 @@ class TestProxyGet(base.TestCase):
         super(TestProxyGet, self).setUp()
 
         self.session = mock.Mock()
+        self.session._sdk_connection = self.cloud
 
         self.fake_id = 1
         self.fake_name = "fake_name"
         self.fake_result = "fake_result"
         self.res = mock.Mock(spec=RetrieveableResource)
         self.res.id = self.fake_id
-        self.res.get = mock.Mock(return_value=self.fake_result)
+        self.res.fetch = mock.Mock(return_value=self.fake_result)
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
         RetrieveableResource.new = mock.Mock(return_value=self.res)
 
     def test_get_resource(self):
         rv = self.sot._get(RetrieveableResource, self.res)
 
-        self.res.get.assert_called_with(self.sot, requires_id=True,
-                                        error_message=mock.ANY)
+        self.res.fetch.assert_called_with(
+            self.sot, requires_id=True,
+            base_path=None,
+            error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_resource_with_args(self):
@@ -316,24 +353,39 @@ class TestProxyGet(base.TestCase):
         rv = self.sot._get(RetrieveableResource, self.res, **args)
 
         self.res._update.assert_called_once_with(**args)
-        self.res.get.assert_called_with(self.sot, requires_id=True,
-                                        error_message=mock.ANY)
+        self.res.fetch.assert_called_with(
+            self.sot, requires_id=True, base_path=None,
+            error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_id(self):
         rv = self.sot._get(RetrieveableResource, self.fake_id)
 
-        RetrieveableResource.new.assert_called_with(id=self.fake_id)
-        self.res.get.assert_called_with(self.sot, requires_id=True,
-                                        error_message=mock.ANY)
+        RetrieveableResource.new.assert_called_with(
+            connection=self.cloud, id=self.fake_id)
+        self.res.fetch.assert_called_with(
+            self.sot, requires_id=True, base_path=None,
+            error_message=mock.ANY)
+        self.assertEqual(rv, self.fake_result)
+
+    def test_get_base_path(self):
+        base_path = 'dummy'
+        rv = self.sot._get(RetrieveableResource, self.fake_id,
+                           base_path=base_path)
+
+        RetrieveableResource.new.assert_called_with(
+            connection=self.cloud, id=self.fake_id)
+        self.res.fetch.assert_called_with(
+            self.sot, requires_id=True, base_path=base_path,
+            error_message=mock.ANY)
         self.assertEqual(rv, self.fake_result)
 
     def test_get_not_found(self):
-        self.res.get.side_effect = exceptions.NotFoundException(
+        self.res.fetch.side_effect = exceptions.ResourceNotFound(
             message="test", http_status=404)
 
         self.assertRaisesRegex(
-            exceptions.NotFoundException,
+            exceptions.ResourceNotFound,
             "test", self.sot._get, RetrieveableResource, self.res)
 
 
@@ -348,15 +400,17 @@ class TestProxyList(base.TestCase):
         self.fake_response = [resource.Resource()]
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
         ListableResource.list = mock.Mock()
         ListableResource.list.return_value = self.fake_response
 
-    def _test_list(self, paginated):
-        rv = self.sot._list(ListableResource, paginated=paginated, **self.args)
+    def _test_list(self, paginated, base_path=None):
+        rv = self.sot._list(ListableResource, paginated=paginated,
+                            base_path=base_path, **self.args)
 
         self.assertEqual(self.fake_response, rv)
         ListableResource.list.assert_called_once_with(
-            self.sot, paginated=paginated, **self.args)
+            self.sot, paginated=paginated, base_path=base_path, **self.args)
 
     def test_list_paginated(self):
         self._test_list(True)
@@ -364,6 +418,9 @@ class TestProxyList(base.TestCase):
     def test_list_non_paginated(self):
         self._test_list(False)
 
+    def test_list_override_base_path(self):
+        self._test_list(False, base_path='dummy')
+
 
 class TestProxyHead(base.TestCase):
 
@@ -371,6 +428,7 @@ class TestProxyHead(base.TestCase):
         super(TestProxyHead, self).setUp()
 
         self.session = mock.Mock()
+        self.session._sdk_connection = self.cloud
 
         self.fake_id = 1
         self.fake_name = "fake_name"
@@ -380,56 +438,48 @@ class TestProxyHead(base.TestCase):
         self.res.head = mock.Mock(return_value=self.fake_result)
 
         self.sot = proxy.Proxy(self.session)
+        self.sot._connection = self.cloud
         HeadableResource.new = mock.Mock(return_value=self.res)
 
     def test_head_resource(self):
         rv = self.sot._head(HeadableResource, self.res)
 
-        self.res.head.assert_called_with(self.sot)
+        self.res.head.assert_called_with(self.sot, base_path=None)
+        self.assertEqual(rv, self.fake_result)
+
+    def test_head_resource_base_path(self):
+        base_path = 'dummy'
+        rv = self.sot._head(HeadableResource, self.res, base_path=base_path)
+
+        self.res.head.assert_called_with(self.sot, base_path=base_path)
         self.assertEqual(rv, self.fake_result)
 
     def test_head_id(self):
         rv = self.sot._head(HeadableResource, self.fake_id)
 
-        HeadableResource.new.assert_called_with(id=self.fake_id)
-        self.res.head.assert_called_with(self.sot)
+        HeadableResource.new.assert_called_with(
+            connection=self.cloud, id=self.fake_id)
+        self.res.head.assert_called_with(self.sot, base_path=None)
         self.assertEqual(rv, self.fake_result)
 
 
-class TestProxyWaits(base.TestCase):
+class TestExtractName(base.TestCase):
 
-    def setUp(self):
-        super(TestProxyWaits, self).setUp()
+    scenarios = [
+        ('slash_servers_bare', dict(url='/servers', parts=['servers'])),
+        ('slash_servers_arg', dict(url='/servers/1', parts=['servers'])),
+        ('servers_bare', dict(url='servers', parts=['servers'])),
+        ('servers_arg', dict(url='servers/1', parts=['servers'])),
+        ('networks_bare', dict(url='/v2.0/networks', parts=['networks'])),
+        ('networks_arg', dict(url='/v2.0/networks/1', parts=['networks'])),
+        ('tokens', dict(url='/v3/tokens', parts=['tokens'])),
+        ('discovery', dict(url='/', parts=['discovery'])),
+        ('secgroups', dict(
+            url='/servers/1/os-security-groups',
+            parts=['servers', 'os-security-groups'])),
+    ]
 
-        self.session = mock.Mock()
-        self.sot = proxy.Proxy(self.session)
+    def test_extract_name(self):
 
-    @mock.patch("openstack.resource.wait_for_status")
-    def test_wait_for(self, mock_wait):
-        mock_resource = mock.Mock()
-        mock_wait.return_value = mock_resource
-        self.sot.wait_for_status(mock_resource, 'ACTIVE')
-        mock_wait.assert_called_once_with(
-            self.sot, mock_resource, 'ACTIVE', [], 2, 120)
-
-    @mock.patch("openstack.resource.wait_for_status")
-    def test_wait_for_params(self, mock_wait):
-        mock_resource = mock.Mock()
-        mock_wait.return_value = mock_resource
-        self.sot.wait_for_status(mock_resource, 'ACTIVE', ['ERROR'], 1, 2)
-        mock_wait.assert_called_once_with(
-            self.sot, mock_resource, 'ACTIVE', ['ERROR'], 1, 2)
-
-    @mock.patch("openstack.resource.wait_for_delete")
-    def test_wait_for_delete(self, mock_wait):
-        mock_resource = mock.Mock()
-        mock_wait.return_value = mock_resource
-        self.sot.wait_for_delete(mock_resource)
-        mock_wait.assert_called_once_with(self.sot, mock_resource, 2, 120)
-
-    @mock.patch("openstack.resource.wait_for_delete")
-    def test_wait_for_delete_params(self, mock_wait):
-        mock_resource = mock.Mock()
-        mock_wait.return_value = mock_resource
-        self.sot.wait_for_delete(mock_resource, 1, 2)
-        mock_wait.assert_called_once_with(self.sot, mock_resource, 1, 2)
+        results = proxy._extract_name(self.url)
+        self.assertEqual(self.parts, results)
